﻿Public Class BackNavigationHelper
    Private _frame As PhoneApplicationFrame
    Private _backNavigationUri As Uri

    Shared Sub New()
        AddHandler PhoneApplicationService.Current.Deactivated, AddressOf OnPhoneActivationServiceDeactivated
        AddHandler PhoneApplicationService.Current.Activated, AddressOf OnPhoneApplicationServiceActivated
        AddHandler PhoneApplicationService.Current.Launching, AddressOf OnPhoneApplicationServiceLaunching
    End Sub

    Public Event BackNavigationStarted As EventHandler(Of BackNavigationStartedEventArgs)
    Public Event BackNavigationCompleted As EventHandler(Of BackNavigationCompletedEventArgs)

    Private Shared _instance As BackNavigationHelper

    Public Shared ReadOnly Property Instance As BackNavigationHelper
        Get
            If _instance Is Nothing Then
                _instance = New BackNavigationHelper
            End If
            Return _instance

        End Get
    End Property

    Private _IsInitialized As Boolean
    Public Property IsInitialized As Boolean
        Get
            Return _IsInitialized
        End Get
        Private Set(ByVal value As Boolean)
            _IsInitialized = value
        End Set
    End Property

    Private _IsLoaded As Boolean
    Public Property IsLoaded As Boolean
        Get
            Return _IsLoaded
        End Get
        Private Set(ByVal value As Boolean)
            _IsLoaded = value
        End Set
    End Property

    Public Function Initialize(ByVal frame As PhoneApplicationFrame,
                               ByVal autoJournal As Boolean,
                               ByVal hideFrameOnNav As Boolean,
                               ByVal validateUriBeforeNav As Boolean) As Boolean
        If Not IsInitialized Then
            AddHandler frame.Navigated, AddressOf frame_Navigated
            AddHandler frame.Navigating, AddressOf frame_Navigating
            AddHandler frame.NavigationStopped, AddressOf frame_NavigationStopped
            _frame = frame
            Me.AutoJournal = autoJournal
            Me.HideFrameonBackNav = hideFrameOnNav
            Me.ValidateUriBeforeBackNav = validateUriBeforeNav
            IsInitialized = True

            Return True
        End If
        Return False
    End Function

#Region "State "
    Private Const BackNavHistoryKey = "BackNavigationHelper.History"
    Private Const HideFrameOnBackNavKey = "BackNavigationHelper.HideFrameOnBackNav"
    Private Const AutoJournalKey = "BackNavigationHelper.AutoJournal"
    Private Const ValidateUriBeforeBackNavKey = "BackNavigationHelper.ValidateUriBeforeBackNav"

    Private Shared Sub OnPhoneApplicationServiceLaunching(ByVal sender As Object, ByVal e As LaunchingEventArgs)
        'Instance.Load()
    End Sub

    Private Sub SaveState(ByVal key As String, ByVal value As Object)
        PhoneApplicationService.Current.State(key) = value
    End Sub

    Private Shared Sub OnPhoneActivationServiceDeactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)
        Instance.Save()
    End Sub

    Private Sub Save()
        SaveState(BackNavHistoryKey, journalingStack.ToArray())
        SaveState(HideFrameOnBackNavKey, HideFrameonBackNav)
        SaveState(ValidateUriBeforeBackNavKey, ValidateUriBeforeBackNav)
        SaveState(AutoJournalKey, AutoJournal)
    End Sub

    Private isActivating = False
    Private Sub Load()

        'TODO: Right now we allow duplication of Loading when a chooser that does not 
        'tombstone but does deactivate  

        Diagnostics.Debug.Assert((_IsLoaded = True AndAlso
                                  PhoneApplicationService.Current.StartupMode = StartupMode.Activate) OrElse
                              _IsLoaded = False)

        If PhoneApplicationService.Current.StartupMode = StartupMode.Activate Then

            Diagnostics.Debug.WriteLine("Activated")
            Dim historyentries() = LoadState(Of Uri())(BackNavHistoryKey, Nothing)
            If historyentries IsNot Nothing Then
                journalingStack.Clear()
                For x = historyentries.Length - 1 To 0 Step -1
                    journalingStack.Push(historyentries(x))
                Next x
            End If

            Instance.HideFrameonBackNav = LoadState(Of Boolean)(HideFrameOnBackNavKey, True)
            Instance.AutoJournal = LoadState(Of Boolean)(AutoJournalKey, True)
            Instance.ValidateUriBeforeBackNav = LoadState(Of Boolean)(ValidateUriBeforeBackNavKey, False)
#If DEBUG Then
        Else
            Diagnostics.Debug.WriteLine("Launched")
#End If
        End If
        Me._IsLoaded = True
    End Sub

    Private Function LoadState(Of T)(ByVal key As String, ByVal defaultValue As T) As T
        If PhoneApplicationService.Current.State.ContainsKey(key) Then
            Return CType(PhoneApplicationService.Current.State(key), T)
        End If
        Return defaultValue

    End Function

    Private Property IsTombstoned As Boolean

    Private Shared Sub OnPhoneApplicationServiceActivated(ByVal sender As Object, ByVal e As ActivatedEventArgs)
        Instance.IsTombstoned = True

        'TODO:   This checks if this was truly a tombstone.. 
        'Some choosers do not tombstone the app in full..  they do deactivate but 
        If PhoneApplicationService.Current.StartupMode = StartupMode.Activate Then Instance.Load()
    End Sub

#End Region

    Private _autoJournal As Boolean
    Public Property AutoJournal As Boolean
        Get
            Return _autoJournal
        End Get
        Set(ByVal value As Boolean)
            If _autoJournal <> value Then
                _autoJournal = value

                If Not AutoJournal Then StopJournaling()
            End If
        End Set
    End Property

    Private Sub frame_NavigationStopped(ByVal sender As Object, ByVal e As NavigationEventArgs)
        If IsDoingBackNavigation Then
            StopBackNavigation(False, True)
        End If

        IsNavigationInProgress = False
    End Sub

    Private _IsDoingBackNavigation As Boolean
    Public Property IsDoingBackNavigation As Boolean
        Get
            Return _IsDoingBackNavigation
        End Get
        Private Set(ByVal value As Boolean)
            _IsDoingBackNavigation = value
        End Set
    End Property

    Private journalingStack As New Collections.Generic.Stack(Of Uri)
    Private Sub StopJournaling()
        journalingStack.Clear()
    End Sub

    Public Sub StopBackNavigation(ByVal completed As Boolean, ByVal userInitiated As Boolean)
        OnStopBackNavigation(completed, userInitiated)
    End Sub

    Public Overridable Sub OnStopBackNavigation(ByVal completed As Boolean, ByVal userInitiated As Boolean)

        Dim eh = Me.BackNavigationCompletedEvent
        Dim args = New BackNavigationCompletedEventArgs With {.Destination = _backNavigationUri,
                                                       .NavigationCompleted = completed,
                                                       .UserCancelled = userInitiated}
        If eh IsNot Nothing Then eh(Me, args)

        IsDoingBackNavigation = False
        _backNavigationUri = Nothing

        If HideFrameonBackNav Then _frame.Opacity = frameOpacity

#If DEBUG Then
        GC.Collect()
        GC.Collect()
        GC.Collect()
#End If

    End Sub

    Private _UriInProgress As Uri
    Public Property UriInProgress As Uri
        Get
            Return _UriInProgress
        End Get
        Friend Set(ByVal value As Uri)
            _UriInProgress = value
        End Set
    End Property

    Private _IsNavigationInProgress As Boolean
    Public Property IsNavigationInProgress As Boolean
        Get
            Return _IsNavigationInProgress
        End Get
        Private Set(ByVal value As Boolean)
            _IsNavigationInProgress = value
        End Set
    End Property
    Private frameOpacity As Double

    Public Sub StartBackNavigation(ByVal targetUri As Uri)
        OnStartBackNavigation(targetUri)
    End Sub

    Public Overridable Function OnStartBackNavigation(ByVal targetUri As Uri) As Boolean
        If AutoJournal AndAlso ValidateUriBeforeBackNav Then
            If Not journalingStack.Contains(targetUri) Then Return False
        End If

        Dim start = Me.BackNavigationStartedEvent
        Dim args = New BackNavigationStartedEventArgs With {.Destination = targetUri}
        If start IsNot Nothing Then start(Me, args)

        'if no page objected to a back nav then do it. 
        If Not args.Cancel Then
            IsDoingBackNavigation = True
            _backNavigationUri = targetUri

            If HideFrameonBackNav Then
                frameOpacity = _frame.Opacity
                _frame.Opacity = 0
            End If

            _frame.GoBack()
        End If

        Return Not args.Cancel
    End Function

    Private currentDirection As NavigationMode

    Private Sub frame_Navigating(ByVal sender As Object,
                                 ByVal e As Windows.Navigation.NavigatingCancelEventArgs)

        currentDirection = e.NavigationMode

        UriInProgress = e.Uri
#If DEBUG Then
        Select Case e.NavigationMode
            Case NavigationMode.Back
            Case NavigationMode.New
            Case NavigationMode.Refresh
                Diagnostics.Debug.Assert(False, "trying to learn when this happens")
            Case NavigationMode.Forward
                Diagnostics.Debug.Assert(False, "trying to learn when this happens")
        End Select
#End If

        Me.IsNavigationInProgress = True
    End Sub


    Public Sub SafeNavigate(ByVal targetUri As Uri, Optional ByVal preventCircles As Boolean = False)
        If IsNavigationInProgress Then
            If UriInProgress <> targetUri Then
                _frame.StopLoading()
                Diagnostics.Debug.WriteLine("Stopped Loading a nav")
            Else
                Diagnostics.Debug.WriteLine("Ignore a double click ")
                Return
            End If
        ElseIf targetUri = _frame.Source Then
            Diagnostics.Debug.WriteLine("Ignore fragment navigation")
            Return
        End If

        If preventCircles Then
            If journalingStack.Contains(targetUri) Then Me.StartBackNavigation(targetUri)

        Else
            _frame.Navigate(targetUri)
        End If
    End Sub

    Private Sub frame_Navigated(ByVal sender As Object, ByVal e As Windows.Navigation.NavigationEventArgs)
        Diagnostics.Debug.WriteLine("Navigated to" & e.Uri.ToString)
        If AutoJournal Then
            If currentDirection = NavigationMode.Back Then
                If journalingStack.Count > 0 Then

                    If Not IsTombstoned Then
                        journalingStack.Pop()
                    Else
                        IsTombstoned = False
                    End If
#If DEBUG Then
                    If journalingStack.Count > 0 Then
                        Dim top = journalingStack.Peek()
                        Diagnostics.Debug.Assert(top Is e.Uri)
                    End If
#End If
                    'we have reached pages that were not on our stack. This likely won't happen 
                    'for real apps, but happens for demos when you turn journaling on/off.. 
                Else
                    journalingStack.Push(e.Uri)
                End If
            End If

            If currentDirection = NavigationMode.New Then
                'SKIP the tombstoning URLs 
                If e.Uri Is New Uri("app://external", UriKind.RelativeOrAbsolute) Then
                    IsTombstoned = True
                    Diagnostics.Debug.WriteLine("*** External *** ")
                Else
                    journalingStack.Push(e.Uri)
                End If
            End If
        End If

        If IsDoingBackNavigation Then
            If e.Uri IsNot _backNavigationUri AndAlso _frame.CanGoBack Then
                _frame.GoBack()
                Return
            Else
                StopBackNavigation(True, False)
            End If

        End If
        IsNavigationInProgress = False
    End Sub

    Public Property HideFrameonBackNav As Boolean
    Public Property ValidateUriBeforeBackNav As Boolean

    Public Shared DemoSettings As New Settings

    Public Class Settings
        Public Property HideAppBarOnNavigate As Boolean
        Public Property PreventLoopCreation As Boolean
    End Class
End Class

Public Class BackNavigationStartedEventArgs
    Inherits EventArgs
    Public Destination As Uri
    Public Cancel As Boolean
End Class

Public Class BackNavigationCompletedEventArgs
    Inherits EventArgs
    Public Destination As Uri
    Public NavigationCompleted As Boolean
    Public UserCancelled As Boolean
End Class