﻿' (c) Copyright Microsoft Corporation.
' This source is subject to the Microsoft Public License (Ms-PL).
' Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
' All other rights reserved.

Imports System
Imports System.Diagnostics.CodeAnalysis

'#If WINDOWS_PHONE Then
'Namespace Microsoft.Phone.Controls
'#Else
'Namespace System.Windows.Controls
'#End If
''' <summary>
''' Implements a weak event listener that allows the owner to be garbage
''' collected if its only remaining link is an event handler.
''' </summary>
''' <typeparam name="TInstance">Type of instance listening for the event.</typeparam>
''' <typeparam name="TSource">Type of source for the event.</typeparam>
''' <typeparam name="TEventArgs">Type of event arguments for the event.</typeparam>
<SuppressMessage("Microsoft.Performance",
    "CA1812:AvoidUninstantiatedInternalClasses",
    Justification:="Used as link target in several projects.")>
Friend Class WeakEventListener(Of TInstance As Class, TSource, TEventArgs)
    ''' <summary>
    ''' WeakReference to the instance listening for the event.
    ''' </summary>
    Private _weakInstance As WeakReference

    ''' <summary>
    ''' Gets or sets the method to call when the event fires.
    ''' </summary>
    Public Property OnEventAction As Action(Of TInstance, TSource, TEventArgs)

    ''' <summary>
    ''' Gets or sets the method to call when detaching from the event.
    ''' </summary>
    Public Property OnDetachAction As Action(Of WeakEventListener(Of TInstance, TSource, TEventArgs))

    ''' <summary>
    ''' Initializes a new instances of the WeakEventListener class.
    ''' </summary>
    ''' <param name="instance">Instance subscribing to the event.</param>
    Public Sub New(ByVal instance As TInstance)
        If Nothing Is instance Then Throw New ArgumentNullException("instance")
        _weakInstance = New WeakReference(instance)
    End Sub

    ''' <summary>
    ''' Handler for the subscribed event calls OnEventAction to handle it.
    ''' </summary>
    ''' <param name="source">Event source.</param>
    ''' <param name="eventArgs">Event arguments.</param>
    Public Sub OnEvent(ByVal source As TSource, ByVal eventArgs As TEventArgs)
        Dim target = CType(_weakInstance.Target, TInstance)
        If Nothing IsNot target Then
            'Call registered action
            If Nothing IsNot OnEventAction Then OnEventAction()(target, source, eventArgs)
        Else
            'Detach from event
            Detach()
        End If
    End Sub

    ''' <summary>
    ''' Detaches from the subscribed event.
    ''' </summary>
    Public Sub Detach()
        If Nothing IsNot OnDetachAction Then
            OnDetachAction()(Me)
            OnDetachAction = Nothing
        End If
    End Sub
End Class
'End Namespace